package com.ruoyi.mybasic.controller;


import com.alibaba.fastjson.JSONObject;
import com.ruoyi.common.core.domain.R;
import com.ruoyi.mybasic.common.domain.Archives;
import com.ruoyi.mybasic.service.ArchivesService;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;

/**
 * @ClassName: ArchivesController
 * @BelongsProject: parent
 * @BelongsPackage: com.ruoyi.archives.controller
 * @Author: zhiS_Lou
 * @CreateTime: 2023/10/17 11:02
 */

/**
 * 档案控制层
 * @author Lou_Zs
 */
@RestController
@RequestMapping("/Archives")
@Log4j2
public class
ArchivesController {

    @Autowired
    private HttpServletRequest request;
    @Autowired
    private ArchivesService archivesService;


    /**
     * 查询我的档案
     * @return
     */
    @GetMapping("/findArchivesList")
    public R<Archives> findArchivesList(){
        log.info("功能介绍:获取我的档案详情,请求方式:{},请求路径:{},请求参数:{}",
                request.getMethod(),request.getRequestURI(),"无参数");

        Archives archives = archivesService.findArchivesList();

        log.info("功能介绍:获取我的档案详情,请求方式:{},请求路径:{},响应结果:{}",
                request.getMethod(),request.getRequestURI(), JSONObject.toJSONString(archives));
        return R.ok(archives);
    }

    /**
     * 添加我的档案
     * @param archives
     * @return
     */
    @PostMapping("/insertArchivesAndPicture")
    public R insertArchivesAndPicture(@RequestBody Archives archives){
        log.info("功能介绍:添加我的档案,请求方式:{},请求路径:{},请求参数:{}",
                request.getMethod(),request.getRequestURI(),JSONObject.toJSONString(archives));

        archivesService.insertArchivesAndPicture(archives);

        log.info("功能介绍:添加我的档案,请求方式:{},请求路径:{},响应结果:{}",
                request.getMethod(),request.getRequestURI(),JSONObject.toJSONString(R.ok()));
        return R.ok();
    }


    /**
     * 编辑我的档案
     * @param archives
     * @return
     */
    @PostMapping("/updateArchivesAndPicture")
    public R updateArchivesAndPicture(@RequestBody Archives archives){
        log.info("功能介绍:编辑我的档案,请求方式:{},请求路径:{},请求参数:{}",
                request.getMethod(),request.getRequestURI(),JSONObject.toJSONString(archives));
        archivesService.updateArchivesAndPicture(archives);

        log.info("功能介绍:编辑我的档案,请求方式:{},请求路径:{},响应结果:{}",
                request.getMethod(),request.getMethod(),JSONObject.toJSONString(R.ok()));
        return R.ok();
    }


    /**
     * 根据档案编号删除档案
     * @param id
     * @return
     */
    @PostMapping("/deleteArchivesById/{id}")
    public R deleteArchivesById(@PathVariable String id){
        log.info("功能介绍:根据档案编号删除档案,请求方式:{},请求路径:{},请求参数:{}",
                request.getMethod(),request.getRequestURI(),JSONObject.toJSONString(id));
        archivesService.deleteArchivesById(id);
        log.info("功能介绍:根据档案编号删除档案,请求方式:{},请求路径:{},响应结果:{}",
                request.getMethod(),request.getRequestURI(), JSONObject.toJSONString(R.ok()));
        return R.ok();
    }




}
