package com.ruoyi.mybasic.controller;
import cn.hutool.json.JSONObject;
import com.alipay.api.AlipayApiException;
import com.alipay.api.AlipayClient;
import com.alipay.api.DefaultAlipayClient;
import com.alipay.api.internal.util.AlipaySignature;
import com.alipay.api.request.AlipayTradePagePayRequest;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.ruoyi.common.core.domain.R;
import com.ruoyi.common.core.exception.ServiceException;
import com.ruoyi.common.security.utils.SecurityUtils;
import com.ruoyi.mybasic.api.domain.Purse;
import com.ruoyi.mybasic.api.domain.UserMoneyLogs;
import com.ruoyi.mybasic.common.domain.MoneyWithdraw;
import com.ruoyi.mybasic.common.domain.RechargeLog;
import com.ruoyi.mybasic.common.domain.ailPay.AliPay;
import com.ruoyi.mybasic.common.domain.ailPay.AliPayConfig;
import com.ruoyi.mybasic.common.domain.request.PurseRequest;
import com.ruoyi.mybasic.common.domain.response.PurseResponse;
import com.ruoyi.mybasic.service.PurseServiceTwo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @ClassName PurseControllerTwo
 * @Description 我的钱包模块接口
 * @Author WenHao.Sao
 */

/**
 * 我的钱包小模块接口-->新
 */
//@CrossOrigin(origins = "*")
@RestController
@RequestMapping("/purse")//purse
public class PurseControllerTwo {

    private static final String GATEWAY_URL = "https://openapi-sandbox.dl.alipaydev.com/gateway.do";
    private static final String FORMAT = "JSON";
    private static final String CHARSET = "UTF-8";
    //签名方式
    private static final String SIGN_TYPE = "RSA2";

    @Resource
    private AliPayConfig aliPayConfig;

    @Autowired
    private PurseServiceTwo purseServiceTwo;


    /**
     * 钱包信息接口：
     * 入参：无
     * 出参：钱包对象
     * 说明：获取当前用户id securityUtils.getUserId()
     * 根据userId查询钱包表
     */
    @GetMapping("/purseObject")
    public R<Purse> purseObject(){
        Purse purse = purseServiceTwo.purseObject();
        return R.ok(purse);
    }


    /**
     * 钱包信息接口：
     * 入参：无
     * 出参：余额
     * 说明：获取当前用户id securityUtils.getUserId()
     * 根据userId查询钱包表
     */
    @GetMapping("/getBalance")
    public R<Integer> getBalance(){
        Purse purse = purseServiceTwo.purseObject();
        return R.ok(purse.getBalanceFee());
    }



    /**
     * 充值接口：
     * 入参: 微信/充值宝渠道 充值金额
     * 出参：统一结果集
     * 说明根据入参调用微信/支付宝 api接口
     * 添加一条专门的充值记录表 状态为待处理
     * 封装api接口需要的参数对象（新增充值记录的id 微信/支付宝 充值金额等），把对象给消息队列
     * 消息队列根据消息调用api接口 消息队列根据api接口的返回结果进行不同的操作
     * 根据api接口返回结果消息队列中的操作失败：根据参数中的充值记录id 把对应记录状态改为失败
     * 成功：根据参数中的充值记录id 把对应记录状态改为成功 用户的余额增加 增加一条收支记录表
     */

    @GetMapping("/pay") // &subject=xxx&traceNo=xxx&totalAmount=xxx
    public void pay(AliPay aliPay, HttpServletResponse httpResponse) throws Exception {
        // 1. 创建Client，通用SDK提供的Client，负责调用支付宝的API
        AlipayClient alipayClient = new DefaultAlipayClient(GATEWAY_URL, aliPayConfig.getAppId(),
                aliPayConfig.getAppPrivateKey(), FORMAT, CHARSET, aliPayConfig.getAlipayPublicKey(), SIGN_TYPE);

        // 2. 创建 Request并设置Request参数
        AlipayTradePagePayRequest request = new AlipayTradePagePayRequest();  // 发送请求的 Request类
        request.setNotifyUrl(aliPayConfig.getNotifyUrl());
        request.setReturnUrl(aliPayConfig.getReturnUrl());
        JSONObject bizContent = new JSONObject();
        bizContent.set("out_trade_no", aliPay.getTraceNo());  // 我们自己生成的订单编号
        bizContent.set("total_amount", aliPay.getTotalAmount()); // 订单的总金额
        bizContent.set("subject", aliPay.getSubject());   // 支付的名称
        bizContent.set("product_code", "FAST_INSTANT_TRADE_PAY");  // 固定配置
        request.setBizContent(bizContent.toString());


        UserMoneyLogs userMoneyLogs = new UserMoneyLogs();

        //获取前台传入的一个参数 充值金额
        double totalAmount = aliPay.getTotalAmount();
        double totalAmountOne = (totalAmount * 100);// 1/100

        userMoneyLogs.setUserId(Math.toIntExact(SecurityUtils.getUserId()));
        userMoneyLogs.setAmountIncome((int) totalAmountOne);
        userMoneyLogs.setMoneyTypeName("支付宝充值");
        userMoneyLogs.setStatus(2);
        //调用修改钱包接口
        purseServiceTwo.purseBalanceChange(userMoneyLogs);

        //充值记录表
        RechargeLog rechargeLog = new RechargeLog();
        rechargeLog.setUserId(SecurityUtils.getUserId());
        rechargeLog.setRechargeType("支付宝");
        rechargeLog.setRechargeAmount(totalAmount);
        purseServiceTwo.insertRechargeLog(rechargeLog);

        // 执行请求，拿到响应的结果，返回给浏览器
        String form = "";
        try {
            form = alipayClient.pageExecute(request).getBody(); // 调用SDK生成表单
        } catch (AlipayApiException e) {
            e.printStackTrace();
        }
        httpResponse.setContentType("text/html;charset=" + CHARSET);
        httpResponse.getWriter().write(form);// 直接将完整的表单html输出到页面
        httpResponse.getWriter().flush();
        httpResponse.getWriter().close();

    }


    /**
     * 创建支付宝回调
     *
     * 在上述代码中，我们首先解析支付宝异步通知的参数，并将其转化为一个 Map 对象。然后，我们使用支付宝提供的 SDK 中的 AlipaySignature.rsaCheckV1 方法来验证签名的有效性。
     *
     * 如果签名验证通过，我们可以根据 trade_status 参数的值来判断支付结果。在示例代码中，我们将支付成功的处理逻辑放在了 TRADE_SUCCESS 和 TRADE_FINISHED 的情况下。
     *
     * 在处理订单逻辑时，您可以根据 out_trade_no 参数获取订单号，并进行相应的处理。
     *
     * 最后，根据处理结果，我们返回 success 或 fail 给支付宝，以告知支付宝是否接收到了通知并处理成功。
     *
     * 请注意，以上代码中的 AlipayConfig 类是自定义的配置类，用于存放支付宝的相关配置信息，包括支付宝公钥、字符编码、签名类型等。您需要根据您自己的配置进行相应的修改。
     *
     * 此外，为了接收支付宝的异步通知，您需要将接口地址（如 /alipay/notify）配置为支付宝商户后台中的异步通知地址。
     *
     * 以上是一个基本的支付宝沙箱环境支付完成回调的代码示例和解释。根据您的具体业务需求，您可能需要在代码中添加更多的逻辑来处理订单信息和其他业务操作。
     */
    @PostMapping("/alipay/notify")
    public String alipayNotify(HttpServletRequest request) throws AlipayApiException {
        //解析支付宝异步通知的参数
        Map<String, String[]> parameterMap = request.getParameterMap();
        HashMap<String, String> params = new HashMap<>();
        for (Map.Entry<String, String[]> entry : parameterMap.entrySet()) {
            String key = entry.getKey();
            String[] values = entry.getValue();
            StringBuilder value = new StringBuilder();
            for (int i = 0; i < value.length(); i++) {
                value.append(values[i]);
                if (i != values.length - 1){
                    value.append(",");
                }
            }
            params.put(key,value.toString());
        }

        //验证签名

        boolean verifyResult = AlipaySignature.rsaCertCheckV1(params, aliPayConfig.getAlipayPublicKey(), CHARSET, SIGN_TYPE);
        if (verifyResult){
            //验证通过 , 处理支付结果
            String tradeStatus = params.get("trade_status");
            if ("TRADE_SUCCESS".equals(tradeStatus) || "TRADE_FINSHED".equals(tradeStatus)){
                //支付成功 处理订单逻辑
                String outTradeNo = params.get("out_trade_no");
                //处理订单逻辑
                //fanhui成功响应给支付宝
                return "success";
            }
        }
        //验证不通过或处理失败,返回失败响应给支付宝
        return "fail";
    }




    /**
     * 提现接口：
     * 入参:提现金额 绑定的银行卡id
     * 出参：统一结果集
     * 说明：先进行余额判断，判断账号钱币是否足够 不够则提示用户
     * 添加一条专门的提现记录 状态为待处理
     * 封装api接口需要的参数对象（新增提现记录的id 银行卡id 提现金额等）,把对象给消息队列
     * 消息队列根据消息调用api接口 消息队列根据api接口的返回结果进行不同的操作失败：根据参数中的提现记录id 把对应记录状态改为失败
     * 成功：根据参数中的充值记录id 把对应记录状态改为成功 用户的余额减少 增加一条收支记录表
     */
    @PostMapping("/withdrawPurse")//完成
    public void withdrawPurse(AliPay aliPay){

        Purse purseResponse = purseServiceTwo.purseObject();
        //获取当前用户的余额
        int balanceFee = purseResponse.getBalanceFee();
        //获取前台输入的金额
        double totalAmount = aliPay.getTotalAmount();
        //判断需要提现的金额 * 100 跟当前H币的金额
        if (totalAmount * 100 > balanceFee){
            throw new ServiceException("H币不足,请重新输入");
        }else{

            //获取一个空对象
            UserMoneyLogs userMoneyLogs = new UserMoneyLogs();


            userMoneyLogs.setUserId(Math.toIntExact(SecurityUtils.getUserId()));
            userMoneyLogs.setAmountIncome((int) totalAmount);
            userMoneyLogs.setMoneyTypeName("提现入支付宝");
            userMoneyLogs.setStatus(1);
            //调用修改钱包接口
            purseServiceTwo.purseBalanceChange(userMoneyLogs);

            //提现记录表
            MoneyWithdraw moneyWithdraw = new MoneyWithdraw();
            moneyWithdraw.setUserId(SecurityUtils.getUserId());
            moneyWithdraw.setBankId("支付宝");
            moneyWithdraw.setAmount(totalAmount + "");
            moneyWithdraw.setStatue(1);
            purseServiceTwo.insertWithdrawalLog(moneyWithdraw);

        }
}





    /**
     * 钱包余额改变接口：
     * 入参：对象(userId,钱币数量,收入/支出标识，类型)
     * 出参：统一结果集
     * 说明：如果是支出也要先进行余额判断
     * 根据参数给对应用户钱包的余额进行改变
     * 添加一条收支记录表
     */
    @PostMapping("/purseBalanceChange")
    public R<?> purseBalanceChange(@RequestBody UserMoneyLogs userMoneyLogs){
        purseServiceTwo.purseBalanceChange(userMoneyLogs);
        return R.ok();
    }



    /**
     *添加钱包：
     * 入参：userId
     * 出参：统一结果集
     * 说明：根据userId添加钱包
     */
    @PostMapping("/addPurse/{userId}")
    public R<?> addPurse(@PathVariable Long userId){
        purseServiceTwo.addPurse(userId);
        return R.ok();

    }



    //    @GetMapping("/pay")
//    public void pay(AilPay ailPay , HttpServletResponse httpServletResponse) throws IOException, AlipayApiException {
//       // 1. 创建Client，通用SDK提供的Client，负责调用支付宝的API
//        AlipayClient alipayClient = new DefaultAlipayClient(GATEWAY_URL, alipayConfig.getAppId(),
//                alipayConfig.getAppPrivateKey(), FORMAT, CHARSET, alipayConfig.getAlipayPublicKey(), SIGN_TYPE);
//        // 2. 创建 Request并设置Request参数
//        AlipayTradePagePayRequest request = new AlipayTradePagePayRequest();  // 发送请求的 Request类
//        request.setNotifyUrl(alipayConfig.getNotifyUrl());
//        request.setReturnUrl(alipayConfig.getReturnUrl());
//
//        JSONObject jsonObject = new JSONObject();
//         int alipayNo;//支付宝订单号
//        jsonObject.set("out_trade_no", "111111");  // 我们自己生成的订单编号
//        jsonObject.set("total_amount", 12); // 订单的总金额
//        jsonObject.set("subject","测试");   // 支付的名称
//        jsonObject.set("product_code", "FAST_INSTANT_TRADE_PAY");  // 固定配置
////        jsonObject.set("", ailPay.getStatus());
//        request.setBizContent(jsonObject.toString());
//
//        JSONArray goodsDerail = new JSONArray();
//        JSONObject entries = new JSONObject();
//        entries.set("user_id" , "goodsNo1");
//        entries.set("goods_name","善商品");
//        entries.set("amount",11);
//        entries.set("price",111);
//        goodsDerail.add(entries);
//        jsonObject.set("en_tries",entries);
//        //扩展信息
//        JSONObject entries1 = new JSONObject();
//        entries1.set("sys_service_provider_id","666666");
//        jsonObject.set("entr_ies1",entries1);
//        request.setBizContent(jsonObject.toString());
//        AlipayTradePagePayResponse response = alipayClient.pageExecute(request);
//        if (response.isSuccess()){
//            System.out.println("调用成功");
//        }else{
//            System.out.println("调用失败");
//
//        }
////        //
////        Long userId = SecurityUtils.getUserId();
////        R<PurseResponse> purseResponseR = purseServiceTwo.purseObject(userId);
////        double totalAmount = ailPay.getTotalAmount();
//////        purseResponseR.getData().setBalanceFee(totalAmount * 100);
////        //创建一个新的对象
////        PurseRequest purseRequest = new PurseRequest();
////        //获取到当前的登录用户赋值
////        purseRequest.setUserId(userId + "");
////        //空余额 赋予新值
////        purseRequest.setBalanceFee(totalAmount * 100);
////        //修改余额
////        purseServiceTwo.updPurseHBi(purseRequest);
//
//
//        //执行请求,拿到响应的结果 ,返回给浏览器
//        String form = "";
//        try {
//            form = alipayClient.pageExecute(request).getBody();//调用SDK生成表单
//        } catch (AlipayApiException e) {
//            e.printStackTrace();
//        }                        //设置方式                        UTF-8
//        httpServletResponse.setContentType("text/html;charset=" + CHARSET);
//        httpServletResponse.getWriter().write(form); //直接将完整的的表单生成到html中
//        httpServletResponse.getWriter().flush();
//        httpServletResponse.getWriter().close();
//    }


//    /**
//     * 回调
//     */
//    @PostMapping("/notify")//这里是post接口
//    public String payNotify(HttpServletRequest request){
//        if (request.getParameter("trade_status").equals("TRADE_SUCCUSS")){
//            System.out.println("========支付宝异步回调==========");
//
//            HashMap<String, String> params = new HashMap<>();
//            Map<String, String[]> parameterMap = request.getParameterMap();
//            for (String name : parameterMap.keySet()) {
//                params.put(name,request.getParameter(name));
//            }
//            String outTradeNo = params.get("out_trade_no");
//            String gmtPayment = params.get("gmt_payment");
//            String tradeNo = params.get("trade_no");
//
//            String sign = params.get("trade_no");
//            String contentV1 = AlipaySignature.getSignCheckContentV1(params);
//            try {
//                boolean b = AlipaySignature.rsa256CheckContent(contentV1, sign, aliPayConfig.getAlipayPublicKey(), "UTF-8");//验证签名
//                //支付宝验签
//                if (b){
//                    //验证通过
//                    System.out.println("支付宝交易凭证号: " + params.get("trade_no"));
//                    System.out.println("商户订单号: " + params.get("out_trade_no"));
//                    System.out.println("交易金额: " + params.get("total_amount"));
//
//                    //查询订单
//                    QueryWrapper<Object> queryWrapper = new QueryWrapper<>();
//                    queryWrapper.eq("order_id",outTradeNo);
//                    List<PurseRequest> listR = purseServiceTwo.purseListAll();
//                }
//            } catch (AlipayApiException e) {
//                e.printStackTrace();
//            }
//        }
//        return "success";
//    }

}
