package com.ruoyi.mybasic.service.Impl;


import com.ruoyi.common.security.utils.SecurityUtils;
import com.ruoyi.mybasic.common.domain.Archives;
import com.ruoyi.mybasic.mapper.ArchivesMapper;
import com.ruoyi.mybasic.service.ArchivesService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * @ClassName: ArchivesServiceImpl
 * @BelongsProject: parent
 * @BelongsPackage: com.ruoyi.archives.service.impl
 * @Author: zhiS_Lou
 * @CreateTime: 2023/10/17 11:05
 */

/**
 * 档案实现层
 * @author Lou_Zs
 */
@Service
public class ArchivesServiceImpl implements ArchivesService {

    @Autowired
    private ArchivesMapper archivesMapper;



    /**
     * 查询我的档案列表
     *
     * @return
     */
    @Override
    public Archives findArchivesList() {
        //获取用户id
        Long userId = SecurityUtils.getUserId();
        Archives archives =  archivesMapper.findArchivesList(userId);
        if (null == archives){
            throw new RuntimeException("查无此档案!");
        }
        return archives;
    }

    /**
     * 添加我的档案
     *
     * @param archives
     */
    @Override
    public void insertArchivesAndPicture(Archives archives) {
        //获取用户id
        Long userId = SecurityUtils.getUserId();
        //添加当前用户
        archives.setUserId(Math.toIntExact(userId));
        /**
         * 若前台传输为[]
         */
        //获取图片集合
        List<String> pictureList = archives.getPictureList();
        if (pictureList.size() > 0){
            //将集合拼接为字符串
            String join = String.join(",", pictureList);
            //将拼接好的字符串添加到字段
            archives.setPictureUrl(join);
        }
        //添加档案
        int i = archivesMapper.insertArchives(archives);
        if (i < 0){
            throw new RuntimeException("未添加成功!");
        }
    }

    /**
     * 编辑我的档案
     * @param archives
     */
    @Override
    public void updateArchivesAndPicture(Archives archives) {
        /**
         * 若前台传输为[]
         */
        //获取图片集合
        List<String> pictureList = archives.getPictureList();
        if (pictureList.size() > 0){
            //将集合拼接为字符串
            String join = String.join(",", pictureList);
            //将拼接好的字符串添加到字段
            archives.setPictureUrl(join);
        }
        //编辑档案
        int i  = archivesMapper.updateArchives(archives);
        if (i < 0){
            throw new RuntimeException("编辑无效!");
        }
    }

    /**
     * 根据档案编号删除档案
     * @param id
     */
    @Override
    public void deleteArchivesById(String id) {
        //档案删除
        int i = archivesMapper.deleteArchivesById(id);
        if (i < 0){
            throw new RuntimeException("删除失败!");
        }
    }

}
