package com.ruoyi.mybasic.service.Impl;

import com.ruoyi.mybasic.api.domain.Purse;
import com.ruoyi.mybasic.api.domain.UserMoneyLogs;
import com.ruoyi.mybasic.common.domain.request.RequestInvitationRecord;
import com.ruoyi.mybasic.common.domain.response.PurseResponse;
import com.ruoyi.mybasic.mapper.InvitationMapper;
import com.ruoyi.mybasic.service.InvitationService;
import com.ruoyi.mybasic.service.PurseServiceTwo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;

/**
 * @ClassName InvitationServiceImpl
 * @Description 邀请记录业务层
 * @Author WenHao.Sao
 */
/**
 * 交易实现层
 * @author Lou_Zs
 */
@Service
public class InvitationServiceImpl implements InvitationService {

    /** @noinspection SpringJavaInjectionPointsAutowiringInspection*/
    @Autowired
    private InvitationMapper invitationMapper;
    @Autowired
    private PurseServiceTwo purseServiceTwo;





    /**
     * 新用户注册邀请获利
     * @param requestInvitationRecord
     */
    @Override
    @Transactional //事务
    public void UserInviteTask(RequestInvitationRecord requestInvitationRecord) {
        int i = 0;
        //新增一条邀请记录
        i = invitationMapper.insertInvitationRecord(requestInvitationRecord);
        if (i < 0){
            throw new RuntimeException("该用户未完成邀请");
        }
        //被邀请人id
        Integer userPid = requestInvitationRecord.getUserPid();
        //邀请人id
        Integer userId = requestInvitationRecord.getUserId();

        //修改邀请人钱包 (1.可用余额+300 2.总计收入+300 3.修改时间)
        i = invitationMapper.updateUserPurse(userId);
        if (i < 0){
            throw new RuntimeException("获利金额未增加到可用余额");
        }
        //同时添加钱包变更记录表
        UserMoneyLogs PurseMoneyLogs = new UserMoneyLogs();
        PurseMoneyLogs.setUserId(userId);
        PurseMoneyLogs.setMoneyTypeName("NEW_USER_INVITE");
        PurseMoneyLogs.setStatus(2);
        PurseMoneyLogs.setAmountIncome(300);
        PurseMoneyLogs.setCreateTime(new Date());
        invitationMapper.insertUserMoneyLogs(PurseMoneyLogs);


        //修改被邀请人钱包 (1.可用余额+300 2.总计收入+300 3.修改时间)
        i = invitationMapper.updateUserPurse(userPid);
        if (i < 0){
            throw new RuntimeException("获利金额未增加到可用余额");
        }
        //同时添加钱包变更记录表
        UserMoneyLogs PurseMoneyLogsTwo = new UserMoneyLogs();
        PurseMoneyLogsTwo.setUserId(userPid);
        PurseMoneyLogsTwo.setMoneyTypeName("NEW_USER_INVITE");
        PurseMoneyLogsTwo.setStatus(2);
        PurseMoneyLogsTwo.setAmountIncome(300);
        PurseMoneyLogsTwo.setCreateTime(new Date());
        invitationMapper.insertUserMoneyLogs(PurseMoneyLogsTwo);

    }

    /**
     * 根据处理金额
     * @param userMoneyLogs
     */
    @Override
    @Transactional//事务
    public void updatePurseByLogs(UserMoneyLogs userMoneyLogs) {
        //初始值
        int i = 0;
        //获取钱包的id
        Integer userId = userMoneyLogs.getUserId();

        //具体金额
        Integer amountIncome = userMoneyLogs.getAmountIncome();

        //判断该条记录为支出记录还是为消费记录
        //1:支出
        //2:收入
        Integer status = userMoneyLogs.getStatus();

        //查询钱包当前信息
        Purse purseResponse = purseServiceTwo.purseObject();
        purseResponse.setUpdateTime(new Date());
        if (status == 1){
            //判断余额是否充足
            if (purseResponse.getBalanceFee() < amountIncome){
                throw new RuntimeException("钱包内余额不足!");
            }
            //支出操作
            purseResponse.setBalanceFee(purseResponse.getBalanceFee() - amountIncome);
            purseResponse.setWalletOutcome(purseResponse.getWalletOutcome() + amountIncome);
//            purseResponse.setWalletIncome(purseResponse.getWalletIncome());
        }else{
            //收入操作
            purseResponse.setBalanceFee(purseResponse.getBalanceFee() + amountIncome);
            purseResponse.setWalletIncome(purseResponse.getWalletIncome() + amountIncome);
//            purse.setWalletOutcome(purseResponse.getWalletOutcome());
        }
        //修改钱包余额
        i = invitationMapper.updateUserMoney(purseResponse);
        if (i < 0){
             throw new RuntimeException("钱包余额变更失败!");
        }
        //余额变更后添加记录表
        i = invitationMapper.insertUserMoneyLogs(userMoneyLogs);
        if (i < 0){
            throw new RuntimeException("订单未生效!");
        }
//        清空redis中的钱包余额数据
//        redisService.deleteObject(userId + SysUserPurse.PURSE_NAME);
//        //再次查询钱包修改后的信息
//        PurseResponse NewpurseResponse = purseServiceTwo.purseObject(Long.valueOf(userId));
//        Integer balanceFee = NewpurseResponse.getBalanceFee();
//        //将查到新用户的余额存储到Redis中
//        redisService.setCacheObject(userId + SysUserPurse.PURSE_NAME,balanceFee);
    }
}
