package com.ruoyi.mybasic.service.Impl;

import com.ruoyi.common.core.exception.ServiceException;
import com.ruoyi.common.security.utils.SecurityUtils;
import com.ruoyi.mybasic.api.domain.Purse;
import com.ruoyi.mybasic.api.domain.UserMoneyLogs;
import com.ruoyi.mybasic.common.domain.MoneyWithdraw;
import com.ruoyi.mybasic.common.domain.RechargeLog;
import com.ruoyi.mybasic.common.domain.request.PurseRequest;
import com.ruoyi.mybasic.common.domain.response.PurseResponse;
import com.ruoyi.mybasic.mapper.PurseMapperTwo;
import com.ruoyi.mybasic.service.PurseServiceTwo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * @ClassName PurseServiceTwoImpl
 * @Description 描述
 * @Author WenHao.Sao
 */

/***
 * 钱包实现层
 */
@Service
public class PurseServiceTwoImpl implements PurseServiceTwo {

    /** @noinspection SpringJavaInjectionPointsAutowiringInspection*/
    @Autowired
    private PurseMapperTwo purseMapperTwo;


    /**
     * 根据当前登录的用户userId
     * 获取到当前钱包对象
     * @param
     * @return
     */
    @Override
    public Purse purseObject() {
        //获取当前用户id
        Long userId = SecurityUtils.getUserId();
        return purseMapperTwo.purseObject(userId);
    }


    /**
     * 添加充值记录
     * @param rechargeLog
     */
    @Override
    public void insertRechargeLog(RechargeLog rechargeLog) {
        int i = purseMapperTwo.insertRechargeLog(rechargeLog);
        if (i <1 ){
            throw new ServiceException("系统繁忙");
        }
    }


    /**
     * 提现记录表
     * @param moneyWithdraw
     */
    @Override
    public void insertWithdrawalLog(MoneyWithdraw moneyWithdraw) {
        purseMapperTwo.insertWithdrawalLog(moneyWithdraw);
    }


    /**
     * 钱包余额改变接口：
     * 入参：对象(userId,钱币数量,收入/支出标识，类型)
     * 出参：统一结果集
     * 说明：如果是支出也要先进行余额判断
     * 根据参数给对应用户钱包的余额进行改变
     * 添加一条收支记录表
     */
    @Override
    public void purseBalanceChange(UserMoneyLogs userMoneyLogs) {

        //获取当前用户的余额
        Purse purseResponse = this.purseObject();
        //当前用户的余额
        int balanceFee = purseResponse.getBalanceFee();
            //获取到状态是收入还是支出 1：支出 2 收入
            int status = userMoneyLogs.getStatus();
            //如果是支出 进行入参金额 和 原有金额 进行判断
            if (1 == status && userMoneyLogs.getAmountIncome() > balanceFee) {
                throw new ServiceException("余额不足");
            }
            //支出/收入调用动态sql
            int i = purseMapperTwo.updateUserBalance(userMoneyLogs);
            if (i < 1){
                throw new ServiceException("系统繁忙");
            }
            //添加记录表中
            purseMapperTwo.addMoneyLogs(userMoneyLogs);
    }



    /**
     * 根据userId 添加钱包
     * @param userId
     * @return
     */
    @Override
    public void addPurse(Long userId) {
        Purse purseResponse = purseMapperTwo.purseObject(userId);
        if (purseResponse != null ){
            throw new ServiceException("已有该用户,请重新创建");
        }
        //创建钱包对象
        Purse purse = new Purse();
        purse.setUserId(userId);
        purse.setBalanceFee(0);
        purse.setWalletIncome(0);
        purse.setWalletOutcome(0);
        Integer i = purseMapperTwo.addPurse(purse);
        if (i < 1){
            throw new ServiceException("系统繁忙,请稍后重试");
        }
    }


}
